@@ -90,7 +90,7 @@ class DDEShellManagerInterfaceV1;
 class WindowPickerInterface;
 class VirtualOutputV1;
 class ShellHandler;
-class PrimaryOutputV1;
+class OutputManagerV1;
 class CaptureSourceSelector;
 class treeland_window_picker_v1;
 class IMultitaskView;
@@ -104,16 +104,28 @@ struct wlr_idle_inhibitor_v1;
 struct wlr_output_power_v1_set_mode_event;
 struct wlr_ext_foreign_toplevel_image_capture_source_manager_v1_request;
 
+namespace Treeland {
+class Treeland;
+}
+
 QW_BEGIN_NAMESPACE
 class qw_ext_foreign_toplevel_image_capture_source_manager_v1;
 QW_END_NAMESPACE
 
-struct Session {
+struct Session : QObject {
+    Q_OBJECT
+public:
+    int id = 0;
     uid_t uid = 0;
     WSocket *socket = nullptr;
     WXWayland *xwayland = nullptr;
     quint32 noTitlebarAtom = XCB_ATOM_NONE;
     SurfaceWrapper *lastActivatedSurface = nullptr;
+
+    ~Session();
+
+Q_SIGNALS:
+    void aboutToBeDestroyed();
 };
 
 class Helper : public WSeatEventFilter
@@ -162,7 +174,7 @@ public:
     ShellHandler *shellHandler() const;
     Workspace *workspace() const;
 
-    void init();
+    void init(Treeland::Treeland *treeland);
 
     TogglableGesture *multiTaskViewGesture() const
     {
@@ -189,8 +201,13 @@ public:
 
     void addSocket(WSocket *socket);
     void removeXWayland(WXWayland *xwayland);
-    WXWayland *xwaylandForUid(uid_t uid, bool createIfMissing = true);
-    WSocket *waylandSocketForUid(uid_t uid, bool createIfMissing = true);
+    void removeSession(std::shared_ptr<Session> session);
+    WXWayland *xwaylandForUid(uid_t uid);
+    WSocket *waylandSocketForUid(uid_t uid);
+    std::shared_ptr<Session> sessionForUid(uid_t uid) const;
+    std::shared_ptr<Session> sessionForXWayland(WXWayland *xwayland) const;
+    std::shared_ptr<Session> sessionForSocket(WSocket *socket) const;
+    std::weak_ptr<Session> activeSession() const;
 
     WSocket *defaultWaylandSocket() const;
     WXWayland *defaultXWaylandSocket() const;
@@ -325,15 +342,13 @@ private:
     void updateIdleInhibitor();
     void setNoAnimation(bool noAnimation);
 
-    std::shared_ptr<Session> sessionForUid(uid_t uid) const;
-    std::shared_ptr<Session> sessionForXWayland(WXWayland *xwayland) const;
-    std::shared_ptr<Session> sessionForSocket(WSocket *socket) const;
-    std::shared_ptr<Session> ensureSession(uid_t uid);
-    void updateActiveUserSession(uid_t uid);
+    std::shared_ptr<Session> ensureSession(int id, uid_t uid);
+    void updateActiveUserSession(const QString &username, int id);
     bool isXWaylandClient(WClient *client);
 
     static Helper *m_instance;
     TreelandConfig *m_config = nullptr;
+    Treeland::Treeland *m_treeland = nullptr;
     FpsDisplayManager *m_fpsManager = nullptr;
 
     CurrentMode m_currentMode{ CurrentMode::Normal };
@@ -378,7 +393,7 @@ private:
     WindowManagementV1::DesktopState m_showDesktop = WindowManagementV1::DesktopState::Normal;
     DDEShellManagerInterfaceV1 *m_ddeShellV1 = nullptr;
     VirtualOutputV1 *m_virtualOutput = nullptr;
-    PrimaryOutputV1 *m_primaryOutputV1 = nullptr;
+    OutputManagerV1 *m_outputManagerV1 = nullptr;
     DDMInterfaceV1 *m_ddmInterfaceV1 = nullptr;
 #ifdef EXT_SESSION_LOCK_V1
     WSessionLockManager *m_sessionLockManager = nullptr;
